@@ -1,8 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
+ * Copyright (c) 1999, 2000, 2001, 2002, 2003 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -25,17 +23,38 @@
 #include <config.h>
 #endif
 #include <stdio.h>
+#ifdef STDC_HEADERS
+#include <stdlib.h>
+#endif
 
 #include "die.h"
 
 static int quiet;
+static int verbose;
+static int debug;
+static void (*exit_proc)();
 
 void
 setquiet()
 {
 	quiet = 1;
 }
-
+void
+setverbose()
+{
+	verbose = 1;
+}
+void
+setdebug()
+{
+	debug = 1;
+}
+void
+sethandler(proc)
+	void (*proc)();
+{
+	exit_proc = proc;
+}
 void
 #ifdef HAVE_STDARG_H
 die(const char *s, ...)
@@ -58,6 +77,10 @@ die(s, va_alist)
 		va_end(ap);
 		fputs("\n", stderr);
 	}
+	if (exit_proc)
+		(*exit_proc)();
+	if (debug)
+		abort();
 	exit(1);
 }
 
@@ -84,5 +107,54 @@ die_with_code(int n, s, va_alist)
 		va_end(ap);
 		fputs("\n", stderr);
 	}
+	if (exit_proc)
+		(*exit_proc)();
+	if (debug)
+		abort();
 	exit(n);
 }
+void
+#ifdef HAVE_STDARG_H
+message(const char *s, ...)
+#else
+message(s, va_alist)
+	char *s;
+	va_dcl;
+#endif
+{
+	va_list ap;
+
+	if (!quiet && verbose) {
+#ifdef HAVE_STDARG_H
+		va_start(ap, s);
+#else
+		va_start(ap);
+#endif
+		(void)vfprintf(stderr, s, ap);
+		va_end(ap);
+		fputs("\n", stderr);
+	}
+}
+void
+#ifdef HAVE_STDARG_H
+warning(const char *s, ...)
+#else
+warning(s, va_alist)
+	char *s;
+	va_dcl;
+#endif
+{
+	va_list ap;
+
+	if (!quiet) {
+		fputs("Warning: ", stderr);
+#ifdef HAVE_STDARG_H
+		va_start(ap, s);
+#else
+		va_start(ap);
+#endif
+		(void)vfprintf(stderr, s, ap);
+		va_end(ap);
+		fputs("\n", stderr);
+	}
+}
