@@ -1,8 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001, 2002
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
+ * Copyright (c) 1999, 2000, 2001, 2002 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -22,7 +20,7 @@
  */
 
 /*
- * If find(1) is available the use it to traverse directory tree.
+ * If find(1) is available, then use it to traverse directory tree.
  * Otherwise use dirent(3).
  */
 #ifdef HAVE_CONFIG_H
@@ -52,6 +50,8 @@
 
 #include "gparam.h"
 #include "regex.h"
+
+#include "char.h"
 #include "conf.h"
 #include "die.h"
 #include "find.h"
@@ -62,35 +62,37 @@
 #include "strlimcpy.h"
 
 /*
- * usage of ?findxxx()
+ * usage of find_xxx()
  *
- *	?findopen();
- *	while (path = ?findread()) {
+ *	find_open(NULL);
+ *	while (path = find_read()) {
  *		...
  *	}
- *	?findclose();
+ *	find_close();
  *
  */
-static regex_t	skip_area;
-static regex_t	*skip;			/* regex for skipping units */
-static regex_t	suff_area;
-static regex_t	*suff = &suff_area;	/* regex for suffixes */
+static regex_t skip_area;
+static regex_t *skip;			/* regex for skipping units */
+static regex_t suff_area;
+static regex_t *suff = &suff_area;	/* regex for suffixes */
 static STRBUF *list;
 static int list_count;
 static char **listarray;		/* list for skipping full path */
-static int	opened;
-static int	retval;
+static int opened;
+static int retval;
 
-static void	trim(char *);
-extern int	debug;
+static void trim(char *);
+#ifdef DEBUG
+extern int debug;
+#endif
 /*
  * trim: remove blanks and '\'.
  */
 static void
 trim(s)
-char	*s;
+	char *s;
 {
-	char	*p;
+	char *p;
 
 	for (p = s; *s; s++) {
 		if (isspace(*s))
@@ -110,9 +112,9 @@ char	*s;
 static void
 prepare_source()
 {
-	STRBUF	*sb = strbuf_open(0);
-	char	*sufflist = NULL;
-	int	flags = REG_EXTENDED;
+	STRBUF *sb = strbuf_open(0);
+	char *sufflist = NULL;
+	int flags = REG_EXTENDED;
 
 	/*
 	 * load icase_path option.
@@ -130,15 +132,16 @@ prepare_source()
 		die("short of memory.");
 	trim(sufflist);
 	{
-		char    *suffp;
+		char *suffp;
 
 		strbuf_reset(sb);
 		strbuf_puts(sb, "\\.(");       /* ) */
 		for (suffp = sufflist; suffp; ) {
-			char    *p;
+			char *p;
 
 			for (p = suffp; *p && *p != ','; p++) {
-				strbuf_putc(sb, '\\');
+				if (!isalnum(*p))
+					strbuf_putc(sb, '\\');
 				strbuf_putc(sb, *p);
 			}
 			if (!*p)
@@ -152,8 +155,10 @@ prepare_source()
 		 * compile regular expression.
 		 */
 		retval = regcomp(suff, strbuf_value(sb), flags);
+#ifdef DEBUG
 		if (debug)
 			fprintf(stderr, "find regex: %s\n", strbuf_value(sb));
+#endif
 		if (retval != 0)
 			die("cannot compile regular expression.");
 	}
@@ -175,7 +180,7 @@ prepare_skip()
 	STRBUF *reg = strbuf_open(0);
 	int reg_count = 0;
 	char *p, *q;
-	int     flags = REG_EXTENDED|REG_NEWLINE;
+	int flags = REG_EXTENDED|REG_NEWLINE;
 
 	/*
 	 * load icase_path option.
@@ -211,7 +216,7 @@ prepare_skip()
 	 */
 	strbuf_putc(reg, '(');	/* ) */
 	for (p = skiplist; p; ) {
-		char    *skipf = p;
+		char *skipf = p;
 		if ((p = locatestring(p, ",", MATCH_FIRST)) != NULL)
 			*p++ = 0;
 		if (*skipf == '/') {
@@ -221,7 +226,7 @@ prepare_skip()
 			reg_count++;
 			strbuf_putc(reg, '/');
 			for (q = skipf; *q; q++) {
-				if (*q == '.')
+				if (isregexchar(*q))
 					strbuf_putc(reg, '\\');
 				strbuf_putc(reg, *q);
 			}
@@ -239,8 +244,10 @@ prepare_skip()
 		 */
 		skip = &skip_area;
 		retval = regcomp(skip, strbuf_value(reg), flags);
+#ifdef DEBUG
 		if (debug)
 			fprintf(stderr, "skip regex: %s\n", strbuf_value(reg));
+#endif
 		if (retval != 0)
 			die("cannot compile regular expression.");
 	} else {
@@ -249,20 +256,28 @@ prepare_skip()
 	if (list_count > 0) {
 		int i;
 		listarray = (char **)malloc(sizeof(char *) * list_count);
+		if (listarray == NULL)
+			die("short of memory.");
 		p = strbuf_value(list);
+#ifdef DEBUG
 		if (debug)
 			fprintf(stderr, "skip list: ");
+#endif
 		for (i = 0; i < list_count; i++) {
+#ifdef DEBUG
 			if (debug) {
 				fprintf(stderr, "%s", p);
 				if (i + 1 < list_count)
 					fputc(',', stderr);
 			}
+#endif
 			listarray[i] = p;
 			p += strlen(p) + 1;
 		}
+#ifdef DEBUG
 		if (debug)
 			fputc('\n', stderr);
+#endif
 	}
 	strbuf_close(reg);
 	free(skiplist);
@@ -275,7 +290,7 @@ prepare_skip()
  */
 int
 skipthisfile(path)
-char *path;
+	char *path;
 {
 	char *first, *last;
 	int i;
@@ -308,13 +323,13 @@ char *path;
 }
 #ifndef HAVE_FIND
 /*----------------------------------------------------------------------*/
-/* dirent version findxxx()						*/
+/* dirent version find_xxx()						*/
 /*----------------------------------------------------------------------*/
 #define STACKSIZE 50
-static  char    dir[MAXPATHLEN+1];		/* directory path */
+static  char dir[MAXPATHLEN+1];			/* directory path */
 static  struct {
-	STRBUF  *sb;
-	char    *dirp, *start, *end, *p;
+	STRBUF *sb;
+	char *dirp, *start, *end, *p;
 } stack[STACKSIZE], *topp, *curp;		/* stack */
 
 static int getdirs(char *, STRBUF *);
@@ -332,10 +347,10 @@ static int getdirs(char *, STRBUF *);
  */
 static int
 getdirs(dir, sb)
-char    *dir;
-STRBUF  *sb;
+	char *dir;
+	STRBUF *sb;
 {
-	DIR     *dirp;
+	DIR *dirp;
 	struct dirent *dp;
 #ifndef HAVE_DP_D_TYPE
 	struct stat st;
@@ -362,7 +377,7 @@ STRBUF  *sb;
 		else if (dp->d_type == DT_REG)
 			strbuf_putc(sb, 'f');
 		else if (dp->d_type == DT_LNK)
-			strbuf_putc(sb, 'l');
+			continue;	/* strbuf_putc(sb, 'l'); */
 		else
 			strbuf_putc(sb, ' ');
 		strbuf_puts(sb, dp->d_name);
@@ -384,7 +399,7 @@ STRBUF  *sb;
 			strbuf_putc(sb, 'f');
 #ifdef S_ISLNK
 		else if (S_ISLNK(st.st_mode))
-			strbuf_putc(sb, 'l');
+			continue;	/* strbuf_putc(sb, 'l'); */
 #endif
 		else
 			strbuf_putc(sb, ' ');
@@ -397,20 +412,25 @@ STRBUF  *sb;
 }
 /*
  * find_open: start iterator without GPATH.
+ *
+ *	i)	start	start directory
+ *			If NULL, assumed '.' directory.
  */
 void
-find_open()
+find_open(start)
+	char *start;
 {
 	assert(opened == 0);
 	opened = 1;
 
+	if (!start)
+		start = ".";
 	/*
 	 * setup stack.
 	 */
 	curp = &stack[0];
 	topp = curp + STACKSIZE; 
-	strlimcpy(dir, ".", sizeof(dir));
-
+	strlimcpy(dir, start, sizeof(dir));
 	curp->dirp = dir + strlen(dir);
 	curp->sb = strbuf_open(0);
 	if (getdirs(dir, curp->sb) < 0)
@@ -432,12 +452,12 @@ find_open()
 char    *
 find_read(void)
 {
-	static	char val[MAXPATHLEN+1];
+	static char val[MAXPATHLEN+1];
 
 	for (;;) {
 		while (curp->p < curp->end) {
-			char	type = *(curp->p);
-			char    *unit = curp->p + 1;
+			char type = *(curp->p);
+			char *unit = curp->p + 1;
 
 			curp->p += strlen(curp->p) + 1;
 			if (type == 'f' || type == 'l') {
@@ -456,8 +476,8 @@ find_read(void)
 				return val;
 			}
 			if (type == 'd') {
-				STRBUF  *sb = strbuf_open(0);
-				char    *dirp = curp->dirp;
+				STRBUF *sb = strbuf_open(0);
+				char *dirp = curp->dirp;
 
 				strcat(dirp, "/");
 				strcat(dirp, unit);
@@ -509,21 +529,33 @@ find_close(void)
 /*----------------------------------------------------------------------*/
 /* find command version							*/
 /*----------------------------------------------------------------------*/
-static FILE	*ip;
+static FILE *ip;
 
 /*
  * find_open: start iterator without GPATH.
+ *
+ *	i)	start	start directory
+ *			If NULL, assumed '.' directory.
  */
 void
-find_open()
+find_open(start)
+	char *start;
 {
-	char	*findcom = "find . -type f -print";
-
+	char findcom[MAXFILLEN+1];
 	assert(opened == 0);
 	opened = 1;
 
+	if (!start)
+		start = ".";
+	if (is_unixy())
+		snprintf(findcom, sizeof(findcom), "find %s \\( -type f -o -type l \\) -print", start);
+	else
+		snprintf(findcom, sizeof(findcom), "find %s -type f -print", start);
+
+#ifdef DEBUG
 	if (debug)
 		fprintf(stderr, "find com: %s\n", findcom);
+#endif
 	/*
 	 * prepare regular expressions.
 	 */
@@ -540,9 +572,9 @@ find_open()
 char	*
 find_read(void)
 {
-	static char	val[MAXPATHLEN+2];
-	char	*path = &val[1];
-	char	*p;
+	static char val[MAXPATHLEN+2];
+	char *path = &val[1];
+	char *p;
 
 	assert(opened == 1);
 	while (fgets(path, MAXPATHLEN, ip)) {
@@ -551,7 +583,7 @@ find_read(void)
 		 */
 		p = path + strlen(path) - 1;
 		if (*p != '\n')
-			die("output of find(1) is wrong (findread).");
+			die("output of find(1) is wrong (find_read).");
 		*p = 0;
 		if (skipthisfile(path))
 			continue;
