@@ -1,8 +1,6 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999
- *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000
- *             Tama Communications Corporation. All rights reserved.
+ * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
+ * Copyright (c) 1999, 2000, 2002 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -44,6 +42,7 @@
 #include "gtagsop.h"
 #include "makepath.h"
 #include "path.h"
+#include "strlimcpy.h"
 #include "test.h"
 
 static const char *makeobjdirprefix;	/* obj partition		*/
@@ -54,7 +53,7 @@ static const char *makeobjdir;		/* obj directory		*/
  */
 static void
 setupvariables(verbose)
-int verbose;
+	int verbose;
 {
 	char *p;
 
@@ -88,12 +87,12 @@ int verbose;
  */
 int
 gtagsexist(candidate, dbpath, size, verbose)
-char	*candidate;
-char	*dbpath;
-int	size;
-int	verbose;
+	char *candidate;
+	char *dbpath;
+	int size;
+	int verbose;
 {
-	char	path[MAXPATHLEN+1];
+	char path[MAXPATHLEN+1];
 
 	/*
 	 * setup makeobjdir and makeobjdirprefix (only first time).
@@ -101,57 +100,37 @@ int	verbose;
 	if (makeobjdir == NULL)
 		setupvariables(verbose);
 
-#ifdef HAVE_SNPRINTF
 	snprintf(path, sizeof(path), "%s/%s", candidate, dbname(GTAGS));
-#else
-	sprintf(path, "%s/%s", candidate, dbname(GTAGS));
-#endif /* HAVE_SNPRINTF */
 	if (verbose)
 		fprintf(stderr, "checking %s\n", path);
 	if (test("fr", path)) {
 		if (verbose)
 			fprintf(stderr, "GTAGS found at '%s'.\n", path);
-#ifdef HAVE_SNPRINTF
 		snprintf(dbpath, size, "%s", candidate);
-#else
-		sprintf(dbpath, "%s", candidate);
-#endif /* HAVE_SNPRINTF */
 		return 1;
 	}
-#ifdef HAVE_SNPRINTF
-	snprintf(path, sizeof(path), "%s/%s/%s", candidate, makeobjdir, dbname(GTAGS));
-#else
-	sprintf(path, "%s/%s/%s", candidate, makeobjdir, dbname(GTAGS));
-#endif /* HAVE_SNPRINTF */
+	snprintf(path, sizeof(path),
+		"%s/%s/%s", candidate, makeobjdir, dbname(GTAGS));
 	if (verbose)
 		fprintf(stderr, "checking %s\n", path);
 	if (test("fr", path)) {
 		if (verbose)
 			fprintf(stderr, "GTAGS found at '%s'.\n", path);
-#ifdef HAVE_SNPRINTF
 		snprintf(dbpath, size, "%s/%s", candidate, makeobjdir);
-#else
-		sprintf(dbpath, "%s/%s", candidate, makeobjdir);
-#endif /* HAVE_SNPRINTF */
 		return 1;
 	}
-#ifdef HAVE_SNPRINTF
-	snprintf(path, sizeof(path), "%s%s/%s", makeobjdirprefix, candidate, dbname(GTAGS));
-#else
-	sprintf(path, "%s%s/%s", makeobjdirprefix, candidate, dbname(GTAGS));
-#endif /* HAVE_SNPRINTF */
+#if !defined(_WIN32) && !defined(__DJGPP__)
+	snprintf(path, sizeof(path),
+		"%s%s/%s", makeobjdirprefix, candidate, dbname(GTAGS));
 	if (verbose)
 		fprintf(stderr, "checking %s\n", path);
 	if (test("fr", path)) {
 		if (verbose)
 			fprintf(stderr, "GTAGS found at '%s'.\n", path);
-#ifdef HAVE_SNPRINTF
 		snprintf(dbpath, size, "%s%s", makeobjdirprefix, candidate);
-#else
-		sprintf(dbpath, "%s%s", makeobjdirprefix, candidate);
-#endif /* HAVE_SNPRINTF */
 		return 1;
 	}
+#endif
 	return 0;
 }
 /*
@@ -175,13 +154,13 @@ int	verbose;
  */
 void
 getdbpath(cwd, root, dbpath, verbose)
-char	*cwd;
-char	*root;
-char	*dbpath;
-int	verbose;
+	char *cwd;
+	char *root;
+	char *dbpath;
+	int verbose;
 {
 	struct stat sb;
-	char	*p;
+	char *p;
 
 	if (!getcwd(cwd, MAXPATHLEN))
 		die("cannot get current directory.");
@@ -206,10 +185,10 @@ int	verbose;
 				die("GTAGSDBPATH must be an absolute path.");
 			if (stat(p, &sb) || !S_ISDIR(sb.st_mode))
 				die("directory '%s' not found.", p);
-			strcpy(dbpath, getenv("GTAGSDBPATH"));
+			strlimcpy(dbpath, getenv("GTAGSDBPATH"), MAXPATHLEN);
 		} else {
 			if (!gtagsexist(root, dbpath, MAXPATHLEN, verbose))
-				die("GTAGS not found.");
+				die_with_code(3, "GTAGS not found.");
 		}
 	} else {
 		if (verbose && getenv("GTAGSDBPATH"))
@@ -217,7 +196,7 @@ int	verbose;
 		/*
 		 * start from current directory to '/' directory.
 		 */
-		strcpy(root, cwd);
+		strlimcpy(root, cwd, MAXPATHLEN);
 		p = root + strlen(root);
 		while (!gtagsexist(root, dbpath, MAXPATHLEN, verbose)) {
 			while (*--p != '/' && p > root)
@@ -227,6 +206,43 @@ int	verbose;
 				break;
 		}
 		if (*root == 0)
-			die("GTAGS not found.");
+			die_with_code(3, "GTAGS not found.");
+		/*
+		 * If file 'GTAGSROOT' found without environment variable
+		 * GTAGSDBPATH, use the value of it as GTAGSROOT.
+		 */
+		do {
+			FILE *fp;
+			STRBUF *sb;
+			char *s, *path;
+
+			path = makepath(root, "GTAGSROOT", NULL);
+			if (!test("fr", path)) {
+				break;
+			}
+			fp = fopen(path, "r");
+			if (fp == NULL) {
+				if (verbose)
+					fprintf(stderr, "'%s' ignored because it cannot be opened.\n", path);
+				break;
+			}
+			sb = strbuf_open(0);
+			s = strbuf_fgets(sb, fp, STRBUF_NOCRLF);
+			if (!test("d", s)) {
+				if (verbose)
+					fprintf(stderr, "'%s' ignored because it doesn't include existent directory name.\n", path);
+			} else {
+				if (verbose)
+					fprintf(stderr, "GTAGSROOT found at '%s'.\n", path);
+				if (*s != '/') {
+					char buf[MAXPATHLEN+1];
+					s = realpath(makepath(root, s, NULL), buf);
+				}
+				strlimcpy(root, s, MAXPATHLEN);
+			}
+			fclose(fp);
+			strbuf_close(sb);
+			break;
+		} while (0);
 	}
 }
